package net.seelisoft.anruflaut;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.telephony.TelephonyManager;

import com.google.gson.Gson;

import net.seelisoft.anruflaut.model.Anrufer;
import net.seelisoft.anruflaut.model.AnruferListe;

public class CallReceiver extends BroadcastReceiver {

    private static Ringer ringer = null;
    private final Gson gson = new Gson();

    public CallReceiver() {
        Logger.log("CallReceiver", "constructor");
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        try {
            onReceiveInternal(context, intent);
        } catch (Throwable t) {
            Logger.log("CallReceiver", "Exception in onReceive: " + t.getMessage(), t);
        }
    }

    private void onReceiveInternal(Context context, Intent intent) {
        if (!"android.intent.action.PHONE_STATE".equals(intent.getAction())) {
            Logger.log("CallReceiver", "onReceive, unerwartete Action: " + intent.getAction());
            return;
        }

        String state = intent.getExtras().getString(TelephonyManager.EXTRA_STATE);
        String nummer = intent.getExtras().getString(TelephonyManager.EXTRA_INCOMING_NUMBER);

        Logger.log("CallReceiver", "onReceive: " + state + "/" + nummer);

        SharedPreferences sharedPrefs = context.getSharedPreferences(Constants.SharedPref_ID, Context.MODE_PRIVATE);
        boolean masterSwitch = sharedPrefs.getBoolean(Constants.SharedPref_MasterSwitch, false);
        int volume = sharedPrefs.getInt(Constants.SharedPref_Volume, Constants.VolumeDefault);
        boolean eigenerKlingelton = sharedPrefs.getBoolean(Constants.SharedPref_EigenerKlingelton, false);

        AnrufProtokoll anrufProtokoll = new AnrufProtokoll(sharedPrefs);

        boolean isNrAktiv = istNummerAktiv(nummer, anrufProtokoll, sharedPrefs);
        boolean isLautlos = isLautlos(context);
        boolean isAnruf = TelephonyManager.EXTRA_STATE_RINGING.equals(state);

        Logger.log("CallReceiver", "So siehts aus: " + masterSwitch + "/" + isNrAktiv + "/" + isLautlos);

        if (isAnruf && nummer != null) {
            anrufProtokoll.nrHinzufügen(nummer);
        }

        killRinger();

        if (masterSwitch && isNrAktiv && isAnruf && isLautlos) {
            ringer = new Ringer(context, volume, eigenerKlingelton);
        }
    }

    private boolean istNummerAktiv(String nummer, AnrufProtokoll anrufProtokoll, SharedPreferences sharedPrefs) {
        if (nummer == null) return false;

        String json = sharedPrefs.getString(Constants.SharedPref_Anrufer, "");
        AnruferListe anruferListe = gson.fromJson(json, AnruferListe.class);

        if (anruferListe == null || anruferListe.anrufer == null || anruferListe.anrufer.isEmpty()) {
            return false;
        }

        boolean istZweiterAnruf = anrufProtokoll.enthältNr(nummer);

        for (Anrufer anrufer : anruferListe.anrufer) {
            if (anrufer.istAktiv() && anrufer.getNummer().equals(nummer) && (anrufer.istSofort() || istZweiterAnruf)) {
                return true;
            }
        }

        return false;
    }

    private boolean isLautlos(Context context) {
        AudioManager am = (AudioManager) context.getSystemService(Context.AUDIO_SERVICE);
        return am.getRingerMode() == AudioManager.RINGER_MODE_SILENT || am.getRingerMode() == AudioManager.RINGER_MODE_VIBRATE;
    }

    private void killRinger() {
        if (ringer != null) {
            ringer.stop();
            ringer = null;
        }
    }
}
