package net.seelisoft.anruflaut;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.ContactsContract;
import android.provider.Settings;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SwitchCompat;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.google.gson.Gson;

import net.seelisoft.anruflaut.model.Anrufer;
import net.seelisoft.anruflaut.model.AnruferListe;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;

public class MainActivity extends AppCompatActivity implements ActivityCompat.OnRequestPermissionsResultCallback, SeekBar.OnSeekBarChangeListener {

    private SharedPreferences sharedPrefs;
    private final Gson gson = new Gson();
    private AnruferListe anruferListe;

    private final int RequestCodeKlingelton = 1;
    private final int RequestCodeBerechtigung = 2;
    private final int RequestCodeKontakt = 3;
    private Ringer ringer;
    private int volume;
    private boolean akkuoptimierungAufgerufen = false;

    private ImageButton imageButtonPlay;
    private boolean eigenerKlingelton = false;

    private final ActionPicker actionPicker = new ActionPicker(this);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        sharedPrefs = this.getSharedPreferences(Constants.SharedPref_ID, Context.MODE_PRIVATE);

        try {
            if (sharedPrefs == null) {
                findSwitch(R.id.switchMaster).setChecked(false);
                findSeekBar(R.id.seekBarVolume).setProgress(Constants.VolumeDefault);
                findTextView(R.id.textViewLautstaerkeValue).setText(AsText(Constants.VolumeDefault));
                volume = Constants.VolumeDefault;
            } else {
                boolean masterSwitch = sharedPrefs.getBoolean(Constants.SharedPref_MasterSwitch, false);
                volume = sharedPrefs.getInt(Constants.SharedPref_Volume, Constants.VolumeDefault);

                String json = sharedPrefs.getString(Constants.SharedPref_Anrufer, "");
                anruferListe = gson.fromJson(json, AnruferListe.class);

                findSwitch(R.id.switchMaster).setChecked(masterSwitch);
                findSeekBar(R.id.seekBarVolume).setProgress(volume);
                findTextView(R.id.textViewLautstaerkeValue).setText(AsText(volume));

                eigenerKlingelton = sharedPrefs.getBoolean(Constants.SharedPref_EigenerKlingelton, false);
            }

            refreshAnruferView();

            findSwitch(R.id.switchMaster).setOnCheckedChangeListener((CompoundButton view, boolean isChecked) -> onMasterSwitchCheckedChanged(isChecked));
            findSeekBar(R.id.seekBarVolume).setOnSeekBarChangeListener(this);
            findImageButton(R.id.buttonNeu).setOnClickListener((View view) -> bearbeiteAnrufer(null));
            findImageButton(R.id.buttonKlingelton).setOnClickListener((View view) -> wähleKlingelton());
            updateKlingeltonTextView(null);

            boolean ok = prüfeBerechtigungen();
            if (!ok && findSwitch(R.id.switchMaster).isChecked()) {
                findSwitch(R.id.switchMaster).setChecked(false);
            }

            Utils.ersetzeVersion(R.id.textViewAbout, this);
        } catch (Exception e) {
            Logger.log("MainActivity", "Exception in onCreate", e);
        }
    }

    @Override
    public void onResume() {
        super.onResume();

        if (akkuoptimierungAufgerufen) {
            akkuoptimierungAufgerufen = false;
            prüfePowerManagement();
        }
    }

    private boolean prüfeBerechtigungen() {
        boolean ok1 = prüfeBerechtigung(android.Manifest.permission.READ_PHONE_STATE, R.id.textViewBerechtigung1);
        boolean ok2 = prüfeBerechtigung(android.Manifest.permission.READ_CALL_LOG, R.id.textViewBerechtigung2);

        prüfePowerManagement();

        return ok1 && ok2;
    }
    
    private boolean prüfeBerechtigung(String berechtigung, int viewId) {
        boolean gewährt = ContextCompat.checkSelfPermission(this, berechtigung) == PackageManager.PERMISSION_GRANTED;
        TextView view = findTextView(viewId);

        if (gewährt) {
            view.setVisibility(View.GONE);
            return true;
        } else {
            view.setVisibility(View.VISIBLE);
            String meldung = getString(R.string.main_fehlende_berechtigung, berechtigung.replace("android.permission. ", ""));
            view.setText(meldung);
            view.setOnClickListener((View v) -> ActivityCompat.requestPermissions(this, new String[] { berechtigung }, RequestCodeBerechtigung));
            return false;
        }
    }

    private void prüfePowerManagement() {
        boolean ok = true;
        TextView view = findTextView(R.id.textViewAkkuoptimierung);

        PowerManager pm = (PowerManager) this.getSystemService(POWER_SERVICE);
        if (pm != null && !pm.isIgnoringBatteryOptimizations(this.getPackageName())) {
            ok = false;
        }

        if (ok) {
            view.setVisibility(View.GONE);
        } else {
            view.setVisibility(View.VISIBLE);
            view.setText(R.string.main_akku_optimierung);
            view.setOnClickListener((View v) -> {
                akkuoptimierungAufgerufen = true;
                Intent intent = new Intent(Settings.ACTION_IGNORE_BATTERY_OPTIMIZATION_SETTINGS);
                startActivity(intent);
            });
        }
    }

    private void updateKlingeltonTextView(View dialogView) {
        int textId = eigenerKlingelton ? R.string.main_eigener_klingelton : R.string.main_mitgelieferter_klingelton;
        findTextView(R.id.textViewKlingelton).setText(textId);

        if (dialogView != null) {
            ((TextView) dialogView.findViewById(R.id.textViewAuswahl)).setText(textId);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        stopRinger();
    }

    private void refreshAnruferView() {
        LinearLayout anruferView = findViewById(R.id.anruferContainer);

        anruferView.removeAllViews();

        if (anruferListe != null && anruferListe.anrufer != null) {
            for (Anrufer anrufer : anruferListe.anrufer) {
                SwitchCompat sw = new SwitchCompat(this);
                sw.setText(anrufer.getDisplayText());
                sw.setChecked(anrufer.istAktiv());
                sw.setTag(anrufer);
                sw.setTextAppearance(R.style.TextAppearance_AppCompat_Small);
                sw.setOnCheckedChangeListener(this::onAnruferCheckedChanged);
                sw.setLongClickable(true);
                sw.setOnLongClickListener(this::onAnruferLongClick);
                LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
                lp.topMargin = 20;
                lp.bottomMargin = 20;
                sw.setLayoutParams(lp);

                anruferView.addView(sw);
            }
        }
    }

    private void onMasterSwitchCheckedChanged(boolean isChecked) {
        if (isChecked && !prüfeBerechtigungen()) {
            findSwitch(R.id.switchMaster).setChecked(false);
            return;
        }

        SharedPreferences.Editor editor = sharedPrefs.edit();
        editor.putBoolean(Constants.SharedPref_MasterSwitch, isChecked);
        editor.apply();
    }

    private void speichereAnruferListe() {
        SharedPreferences.Editor editor = sharedPrefs.edit();
        String json = gson.toJson(anruferListe);
        editor.putString(Constants.SharedPref_Anrufer, json);
        editor.apply();
    }

    private void onAnruferCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        // Aktiv-Flag setzen auf dem Aufrufer-Objekt, dass auch im Tag gespeichert ist
        ((Anrufer) buttonView.getTag()).aktiv = isChecked;
        speichereAnruferListe();
    }

    private boolean onAnruferLongClick(View view) {
        Anrufer anrufer = (Anrufer) view.getTag();
        bearbeiteAnrufer(anrufer);
        return true;
    }

    public void bearbeiteAnrufer(final Anrufer anrufer) {
        View dialogView = LayoutInflater.from(this).inflate(R.layout.anrufer_edit_dialog, null);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this)
                .setView(dialogView)
                .setTitle(anrufer != null ? R.string.anrufer_edit_bearbeiten : R.string.anrufer_edit_hinzufügen);

        if (anrufer != null) {
            ((EditText) dialogView.findViewById(R.id.dialogNameEditText)).setText(anrufer.name);
            ((EditText) dialogView.findViewById(R.id.dialogNummerEditText)).setText(anrufer.nummer);
            ((SwitchCompat) dialogView.findViewById(R.id.dialogVerzoegertSwitch)).setChecked(anrufer.verzögert);
            dialogView.findViewById(R.id.dialogEntfernenButton).setVisibility(View.VISIBLE);
        } else {
            dialogView.findViewById(R.id.dialogEntfernenButton).setVisibility(View.GONE);
        }

        AlertDialog alertDialog = dialogBuilder.show();

        dialogView.findViewById(R.id.dialogOkButton).setOnClickListener(view -> {
            Anrufer neu = anrufer;

            if (neu == null) {
                if (anruferListe == null) {
                    anruferListe = new AnruferListe();
                }
                if (anruferListe.anrufer == null) {
                    anruferListe.anrufer = new ArrayList<>();
                }

                neu = new Anrufer();
                anruferListe.anrufer.add(neu);
            }

            neu.name = ((EditText) dialogView.findViewById(R.id.dialogNameEditText)).getText().toString();
            neu.nummer = ((EditText) dialogView.findViewById(R.id.dialogNummerEditText)).getText().toString();
            neu.verzögert = ((SwitchCompat) dialogView.findViewById(R.id.dialogVerzoegertSwitch)).isChecked();
            speichereAnruferListe();
            refreshAnruferView();
            alertDialog.dismiss();
        });

        dialogView.findViewById(R.id.dialogAbbrechenButton).setOnClickListener(view -> alertDialog.dismiss());
        dialogView.findViewById(R.id.dialogEntfernenButton).setOnClickListener(view -> {
            anruferListe.anrufer.remove(anrufer);
            speichereAnruferListe();
            refreshAnruferView();
            alertDialog.dismiss();
        });

        dialogView.findViewById(R.id.imageViewKontakt).setOnClickListener(view -> {
            Intent intent = new Intent(Intent.ACTION_PICK);
            intent.setType(ContactsContract.CommonDataKinds.Phone.CONTENT_TYPE);
            actionPicker.start(intent, RequestCodeKontakt, (resultCode, resultData) -> {
                Uri data = resultData.getData();
                Cursor c = getContentResolver().query(data, null, null, null, null);
                if (c.moveToFirst()) {
                    int columnName = c.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME);
                    int columnNummer = c.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER);
                    String name = columnName == -1 ? null : c.getString(columnName);
                    String nummer = columnNummer == -1 ? null : c.getString(columnNummer);

                    if (name != null && !name.trim().equals("") && nummer != null && !nummer.trim().equals("")) {
                        ((EditText) dialogView.findViewById(R.id.dialogNameEditText)).setText(name);
                        ((EditText) dialogView.findViewById(R.id.dialogNummerEditText)).setText(nummer);
                    }
                }
                c.close();
            });
        });
    }

    @Override
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
        volume = findSeekBar(R.id.seekBarVolume).getProgress();
        findTextView(R.id.textViewLautstaerkeValue).setText(AsText(volume));

        SharedPreferences.Editor editor = sharedPrefs.edit();
        editor.putInt(Constants.SharedPref_Volume, volume);
        editor.apply();
    }

    @Override
    public void onStartTrackingTouch(SeekBar seekBar) {
        // noop
    }

    @Override
    public void onStopTrackingTouch(SeekBar seekBar) {
        // noop
    }

    private String AsText(int value) {
        return value + "";
    }

    private void wähleKlingelton() {
        View dialogView = LayoutInflater.from(this).inflate(R.layout.audio_auswahl_dialog, null);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this)
                .setView(dialogView)
                .setTitle(R.string.edit_klingelton_klingelton);

        AlertDialog alertDialog = dialogBuilder.show();

        dialogView.findViewById(R.id.buttonOk).setOnClickListener(view -> alertDialog.dismiss());

        dialogView.findViewById(R.id.radioButtonInterner).setOnClickListener(view -> {
            eigenerKlingelton = false;
            updateKlingeltonTextView(dialogView);
        });

        dialogView.findViewById(R.id.radioButtonEigener).setOnClickListener(view -> {
            Intent intent = new Intent();
            intent.setType("audio/*");
            intent.setAction(Intent.ACTION_OPEN_DOCUMENT);
            actionPicker.start(intent, RequestCodeKlingelton, (resultCode, resultData) -> {
                if (resultCode == Activity.RESULT_OK) {
                    Uri uri = resultData.getData();

                    try {
                        InputStream inputStream = getContentResolver().openInputStream(uri);
                        FileOutputStream outputStream = openFileOutput(Constants.eigenerKlingeltonFilename, Context.MODE_PRIVATE);

                        byte[] buffer = new byte[1024];
                        int len;

                        while ((len = inputStream.read(buffer)) > 0) {
                            outputStream.write(buffer, 0, len);
                        }
                        inputStream.close();
                        outputStream.close();
                    } catch (Exception e) {
                        Logger.log("MainActivity", "Exception beim Lesen des Files", e);
                    }

                    eigenerKlingelton = true;
                    updateKlingeltonTextView(dialogView);
                } else {
                    eigenerKlingelton = false;
                    updateKlingeltonTextView(dialogView);

                    ((RadioButton) dialogView.findViewById(R.id.radioButtonInterner)).setChecked(true);
                }
            });
        });

        alertDialog.setOnDismissListener((sender) -> {
            imageButtonPlay = null;
            stopRinger();

            SharedPreferences.Editor editor = sharedPrefs.edit();
            editor.putBoolean(Constants.SharedPref_EigenerKlingelton, eigenerKlingelton);
            editor.apply();
        });

        if (eigenerKlingelton) {
            ((RadioButton) dialogView.findViewById(R.id.radioButtonEigener)).setChecked(true);
        } else {
            ((RadioButton) dialogView.findViewById(R.id.radioButtonInterner)).setChecked(true);
        }
        updateKlingeltonTextView(dialogView);

        imageButtonPlay = dialogView.findViewById(R.id.imageButtonPlay);
        imageButtonPlay.setOnClickListener(view -> {
            if (ringer == null) {
                ringer = new Ringer(this, volume, eigenerKlingelton);
                imageButtonPlay.setImageResource(R.drawable.ic_baseline_stop_circle_24);
            } else {
                stopRinger();
            }
        });
    }

    private void stopRinger() {
        if (ringer != null) {
            ringer.stop();
            ringer = null;
        }
        if (imageButtonPlay != null) {
            imageButtonPlay.setImageResource(R.drawable.ic_baseline_play_circle_24);
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent resultData) {
        super.onActivityResult(requestCode, resultCode, resultData);
        actionPicker.handleResult(requestCode, resultCode, resultData);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        prüfeBerechtigungen();
    }

    public void hilfeButtonOnClick(View view) {
        Intent intent = new Intent(this, HilfeActivity.class);
        startActivity(intent);
    }

    private SwitchCompat findSwitch(int id) {
        return findViewById(id);
    }

    private SeekBar findSeekBar(int id) {
        return findViewById(id);
    }

    private TextView findTextView(int id) {
        return findViewById(id);
    }

    private ImageButton findImageButton(int id) {
        return findViewById(id);
    }
}
